PRINT 'CREATING DEPENDDB DATABASE CLEANUP'

-- Drop procedures
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'Cleanup') AND type in (N'P')) DROP PROCEDURE Cleanup;
GO

-- Cleanup
-- Procedure: Cleans up all unnecessary data
CREATE PROCEDURE Cleanup AS
BEGIN
	BEGIN TRY
		EXEC SetDeletionInfo 'Cleanup'
	
		RAISERROR('Cleanup process started!', 10, 1) WITH NOWAIT
	
		SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
		
		BEGIN TRANSACTION Cleanup
		
		-- Delete all PVs with RelevancyStatus = Cleanupable
		DECLARE @cleanupableProjectVersions ListOfInt
		INSERT @cleanupableProjectVersions (n) SELECT Id
			FROM ProjectVersion
			WHERE RelevancyStatus = dbo.RelevancyStatus_Cleanupable()

		EXEC dbo.DeleteProjectVersions @cleanupableProjectVersions
		
		-- Delete all PBs which have no associated PVs anymore
		DELETE FROM ProjectBranch WHERE NOT EXISTS (
			SELECT * FROM ProjectVersion pv WHERE pv.ProjectBranchId = ProjectBranch.Id
		)
		
		COMMIT TRANSACTION Cleanup
		
		RAISERROR('Cleanup process finished successfully!', 10, 1) WITH NOWAIT
		
		EXEC ClearDeletionInfo
	END TRY
	BEGIN CATCH
		RAISERROR('Error occurred during cleanup process - rolling back transaction!', 10, 1) WITH NOWAIT
		ROLLBACK TRANSACTION Cleanup
		SELECT 
			ERROR_NUMBER() as ErrorNumber,
			ERROR_SEVERITY() as ErrorSeverity,
			ERROR_STATE() as ErrorState,
			ERROR_PROCEDURE() as ErrorProcedure,
			ERROR_LINE() as ErrorLine,
			ERROR_MESSAGE() as ErrorMessage
	END CATCH
END
GO
