PRINT 'CREATING DEPENDDB DELETION LOG'

-- Drop functions
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_SchemaName') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION DL_SchemaName;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_TableSuffix') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION DL_TableSuffix;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_TriggerSuffix') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION DL_TriggerSuffix;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_FileGroupName') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION DL_FileGroupName;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_FileName') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION DL_FileName;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_DataTableNames') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION DL_DataTableNames;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_DeletionTableNames') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION DL_DeletionTableNames;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'GetMirrorTableName') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION GetMirrorTableName;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'GetTriggerName') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION GetTriggerName;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'GetColumns') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION GetColumns;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'GetDeletionInfo') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION GetDeletionInfo;
GO

-- Drop procedures
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'SetDeletionInfo') AND type in (N'P')) DROP PROCEDURE SetDeletionInfo;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'ClearDeletionInfo') AND type in (N'P')) DROP PROCEDURE ClearDeletionInfo;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DestroyDeletionLogging') AND type in (N'P')) DROP PROCEDURE DestroyDeletionLogging;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'SetupDeletionLogging') AND type in (N'P')) DROP PROCEDURE SetupDeletionLogging;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'RecoverRows') AND type in (N'P')) DROP PROCEDURE RecoverRows;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'RecoverAll') AND type in (N'P')) DROP PROCEDURE RecoverAll;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'CleanupDeletionLog') AND type in (N'P')) DROP PROCEDURE CleanupDeletionLog;
GO

-- Constants that are stored in the DL_Settings table
CREATE FUNCTION DL_SchemaName()
RETURNS SYSNAME 
AS
BEGIN
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_Settings') AND type in (N'U')) 
		RETURN NULL
		
	DECLARE @schemaName SYSNAME
	SELECT @schemaName = SchemaName
	FROM dbo.DL_Settings
	
	RETURN @schemaName
END
GO

CREATE FUNCTION DL_TableSuffix()
RETURNS SYSNAME 
AS
BEGIN
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_Settings') AND type in (N'U')) 
		RETURN NULL

	DECLARE @tableSuffix SYSNAME
	SELECT @tableSuffix = TableSuffix
	FROM dbo.DL_Settings
	
	RETURN @tableSuffix
END
GO

CREATE FUNCTION DL_TriggerSuffix()
RETURNS SYSNAME 
AS
BEGIN
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_Settings') AND type in (N'U')) 
		RETURN NULL

	DECLARE @triggerSuffix SYSNAME
	SELECT @triggerSuffix = TriggerSuffix
	FROM dbo.DL_Settings
	
	RETURN @triggerSuffix
END
GO

CREATE FUNCTION DL_FileGroupName()
RETURNS SYSNAME 
AS
BEGIN
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_Settings') AND type in (N'U')) 
		RETURN NULL

	DECLARE @fileGroupName SYSNAME
	SELECT @fileGroupName = FileGroupName
	FROM dbo.DL_Settings
	
	RETURN @fileGroupName
END
GO

CREATE FUNCTION DL_FileName()
RETURNS SYSNAME 
AS
BEGIN
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_Settings') AND type in (N'U')) 
		RETURN NULL

	DECLARE @fileName SYSNAME
	SELECT @fileName = DB_NAME() + '_' + LEFT(FileGroupName,1) + LOWER(SUBSTRING(FileGroupName,2,LEN(FileGroupName)))
	FROM dbo.DL_Settings
	
	RETURN @fileName
END
GO

CREATE FUNCTION DL_DataTableNames()
RETURNS @returnTable TABLE (
	Name SYSNAME NOT NULL
)
AS
BEGIN
	
	DECLARE @result TABLE (
		schemaName SYSNAME NOT NULL,
		tableName SYSNAME NOT NULL,
		[Level] INT NOT NULL
	);

	WITH fk_tables AS (
		SELECT s1.name AS from_schema, o1.Name AS from_table, s2.name AS to_schema, o2.Name AS to_table
		FROM sys.foreign_keys fk
		INNER JOIN sys.objects o1 ON fk.parent_object_id = o1.object_id
		INNER JOIN sys.schemas s1 ON o1.schema_id = s1.schema_id
		INNER JOIN sys.objects o2 ON fk.referenced_object_id = o2.object_id
		INNER JOIN sys.schemas s2 ON o2.schema_id = s2.schema_id
		--For the purposes of finding dependency hierarchy 
		--we're not worried about self-referencing tables
		WHERE NOT ( s1.name = s2.name  AND o1.name = o2.name )
	)

	, ordered_tables AS (
		SELECT s.name AS schemaName, t.name AS tableName, 0 AS [Level]
		FROM (
			SELECT *
			FROM sys.tables 
			WHERE name <> 'sysdiagrams') t
		INNER JOIN sys.schemas s ON t.schema_id = s.schema_id
		LEFT OUTER JOIN fk_tables fk ON s.name = fk.from_schema
		AND t.name = fk.from_table
		WHERE fk.from_schema IS NULL
	
		UNION ALL
	
		SELECT fk.from_schema, fk.from_table, ot.[Level] + 1
		FROM fk_tables fk
		INNER JOIN ordered_tables ot
		ON fk.to_schema = ot.schemaName
		AND fk.to_table = ot.tableName
	)

	INSERT INTO @result ( schemaName, tableName, [Level] )
	SELECT DISTINCT ot.schemaName, ot.tableName, ot.[Level]
	FROM ordered_tables ot
	INNER JOIN (
		SELECT schemaName, tableName, max([Level]) AS maxLevel
		FROM ordered_tables
		GROUP BY schemaName, tableName
	) mx
	ON ot.schemaName = mx.schemaName
	AND ot.tableName = mx.tableName
	AND mx.maxLevel = ot.[Level]
	AND ot.schemaName = 'dbo'
	AND ot.tableName NOT LIKE 'AspNet_%'
	AND ot.tableName != 'DL_Settings'
	ORDER BY [Level] ASC;

	INSERT INTO @returnTable ( Name )
	SELECT tableName FROM @result
		
	RETURN
END
GO

CREATE FUNCTION DL_DeletionTableNames()
RETURNS @returnTable TABLE (
	Name SYSNAME NOT NULL
)
AS
BEGIN
	INSERT INTO @returnTable
	SELECT name
		FROM sys.tables
		WHERE SCHEMA_NAME(schema_id) = dbo.DL_SchemaName()
		
	RETURN
END
GO

-- GetMirrorTableName
-- Function: Returns the name of the table containing the deleted rows of a given table
CREATE FUNCTION GetMirrorTableName(@tableName SYSNAME)
RETURNS SYSNAME
AS
BEGIN
	RETURN @tableName + dbo.DL_TableSuffix()
END
GO

-- GetTriggerName
-- Function: Returns the name of the trigger used for reacting to row deletes for the given table
CREATE FUNCTION GetTriggerName(@tableName SYSNAME)
RETURNS SYSNAME
AS
BEGIN
	RETURN @tableName + dbo.DL_TriggerSuffix()
END
GO

-- GetColumns
-- Function: Returns a comma separated list of column names of a given table
CREATE FUNCTION GetColumns(@tableName SYSNAME)
RETURNS VARCHAR(MAX)
AS
BEGIN
	RETURN (
		SELECT LEFT(Main.Columns, LEN(Main.Columns) - 1)
		FROM (
			SELECT (
				SELECT c.Name + ',' AS [text()]
				FROM sys.columns c
				WHERE c.object_id = OBJECT_ID(@tableName)
				FOR XML PATH('')
			) [Columns]
		) [Main]
	)
END
GO

-- SetDeletionInfo
-- Procedure: Sets the deletion info of the current connection
CREATE PROCEDURE SetDeletionInfo
	@reason VARCHAR(128)
AS
BEGIN
	DECLARE @ctx VARBINARY(128)
	SELECT @ctx = CONVERT(VARBINARY(128), @reason)
	SET CONTEXT_INFO @ctx
END
GO

-- ClearDeletionInfo
-- Procedure: Clear the deletion info of the current connection
CREATE PROCEDURE ClearDeletionInfo
AS
BEGIN
	DECLARE @ctx VARBINARY(128)
	SELECT @ctx = CONVERT(VARBINARY(128), 0x)
	SET CONTEXT_INFO @ctx
END
GO

-- ClearDeletionInfo
-- Function: Returns the deletion info of the current connection
CREATE FUNCTION GetDeletionInfo()
RETURNS VARCHAR(128)
AS
BEGIN
	DECLARE @ctx VARCHAR(128)
	SELECT @ctx = CONVERT(VARCHAR(128), CONTEXT_INFO())
	RETURN @ctx
END
GO

-- DestroyDeletionLogging
-- Procedure: Reverts all the changes made by SetupDeletionLogging
CREATE PROCEDURE DestroyDeletionLogging
AS
BEGIN
	DECLARE @stmt VARCHAR(max)

	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DL_Settings') AND type in (N'U')) RETURN
		
	DECLARE @schemaName SYSNAME = dbo.DL_SchemaName()
	DECLARE @tableSuffix SYSNAME = dbo.DL_TableSuffix()
	DECLARE @triggerSuffix SYSNAME = dbo.DL_TriggerSuffix()
	DECLARE @fileGroupName SYSNAME = dbo.DL_FileGroupName()
	DECLARE @fileName SYSNAME = dbo.DL_FileName()
		
	DECLARE @tableNames CURSOR
	DECLARE @tableName SYSNAME

	-- Delete mirror tables
	SET @tableNames =
		CURSOR FOR
		SELECT name
		FROM DL_DeletionTableNames()

	OPEN @tableNames
	FETCH NEXT FROM @tableNames INTO @tableName

	WHILE @@FETCH_STATUS = 0 BEGIN
		SET @stmt = 'DROP TABLE [' + @schemaName + '].[' + @tableName + ']'
		PRINT(@stmt)
		EXEC(@stmt)
		FETCH NEXT FROM @tableNames INTO @tableName
	END

	CLOSE @tableNames
	DEALLOCATE @tableNames
	
	-- Delete schema
	SET @stmt = 'DROP SCHEMA [' + @schemaName + ']'
	PRINT(@stmt)
	EXEC(@stmt)
	
	SET @stmt = 'DROP TABLE [DL_Settings]'
	PRINT(@stmt)
	EXEC(@stmt)
	
	DECLARE @dbName SYSNAME = DB_NAME()
	
	SET @stmt = 'ALTER DATABASE [' + @dbName + '] REMOVE FILE [' + @fileName + ']'
	PRINT(@stmt)
	EXEC(@stmt) 

	SET @stmt = 'ALTER DATABASE [' + @dbName + '] REMOVE FILEGROUP [FG_' + @fileGroupName + ']'
	PRINT(@stmt)
	EXEC(@stmt)
END
GO

-- SetupDeletionLogging
-- Procedure: Sets up all the necessary infrastructure for deletion logging
CREATE PROCEDURE SetupDeletionLogging
	@schemaName SYSNAME = 'del',
	@tableSuffix SYSNAME = '_Deleted',
	@triggerSuffix SYSNAME = '_AfterDelete',
	@fileGroupName SYSNAME = 'DELETIONLOG'
AS
BEGIN
	DECLARE @stmt VARCHAR(MAX)

	EXEC DestroyDeletionLogging
	
	-- Create settings table
	CREATE TABLE DL_Settings (SchemaName SYSNAME, TableSuffix SYSNAME, TriggerSuffix SYSNAME, FileGroupName SYSNAME)
	INSERT INTO DL_Settings
	SELECT @schemaName as SchemaName, @tableSuffix as TableSuffix, @triggerSuffix as TriggerSuffix, @fileGroupName as FileGroupName
	
	-- Create filegroup
	DECLARE @dbName SYSNAME = DB_NAME()
	DECLARE @fileName SYSNAME = dbo.DL_FileName()
	DECLARE @filePath VARCHAR(MAX)
	
	SET @stmt = 'ALTER DATABASE [' + @dbName + '] ADD FILEGROUP [FG_' + @fileGroupName +']'
	PRINT(@stmt)
	EXEC(@stmt)
	
	SELECT 
		@filePath =
			SUBSTRING(f.physical_name, 0, LEN(f.physical_name) - CHARINDEX('\', REVERSE(f.physical_name)) + 2) +
			@fileName + '.mdf'
	FROM 
		sys.database_files f
		INNER JOIN sys.data_spaces d ON (d.data_space_id = f.data_space_id)
	WHERE d.is_default = 1
	
	SET @stmt = '
		ALTER DATABASE [' + @dbName + '] 
			ADD FILE ( 
				NAME = [' + @fileName +'], 
				FILENAME = '''+@filePath+'''
			) TO FILEGROUP [FG_' + @fileGroupName +']'
	PRINT @stmt
	EXEC(@stmt)

	-- Create schema
	SET @stmt = 'CREATE SCHEMA [' + @schemaName + ']'
	PRINT(@stmt)
	EXEC(@stmt)

	-- Create mirror tables and triggers
	DECLARE @tableNames CURSOR
	DECLARE @tableName SYSNAME

	SET @tableNames =
		CURSOR FOR
		SELECT name
		FROM dbo.DL_DataTableNames()
		
	OPEN @tableNames
	FETCH NEXT FROM @tableNames INTO @tableName

	WHILE @@FETCH_STATUS = 0 BEGIN
		DECLARE @mirrorName SYSNAME
		SELECT @mirrorName = dbo.GetMirrorTableName(@tableName)
		
		DECLARE @triggerName SYSNAME
		SELECT @triggerName = dbo.GetTriggerName(@tableName)
	
		SET @stmt = 'ALTER DATABASE [' + @dbName + '] MODIFY FILEGROUP [FG_' + @fileGroupName +'] DEFAULT'
		PRINT(@stmt)
		EXEC(@stmt)
		
		SET @stmt = 'SELECT *, CONVERT(VARCHAR(128), 0) as DeletionInfo, GETDATE() as DeletionDate INTO [' + @schemaName + '].[' + @mirrorName + '] FROM [dbo].[' + @tableName + '] WHERE 1=2'
		PRINT(@stmt)
		EXEC(@stmt)
		
		SET @stmt = 'ALTER DATABASE [' + @dbName + '] MODIFY FILEGROUP [PRIMARY] DEFAULT'
		PRINT(@stmt)
		EXEC(@stmt)
		
		DECLARE @columns VARCHAR(max)
		SELECT @columns = dbo.GetColumns(@tableName)
		
		SET @stmt = '
			IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N''' + @triggerName + ''') AND type in (N''TR'')) 
				DROP TRIGGER [' + @triggerName + ']'
		PRINT(@stmt)
		EXEC(@stmt)
			
		SET @stmt = '
			CREATE TRIGGER [' + @triggerName + '] ON [dbo].[' + @tableName + '] AFTER DELETE AS
				DECLARE @info VARCHAR(128) = dbo.GetDeletionInfo()
				
				IF EXISTS (SELECT * FROM sys.columns WHERE OBJECT_NAME(object_id) = ''' + @mirrorName + ''' AND is_identity = 1)
					SET IDENTITY_INSERT [' + @schemaName + '].[' + @mirrorName + '] ON
		
				INSERT INTO [' + @schemaName + '].[' + @mirrorName + '] (' + @columns + ', DeletionInfo, DeletionDate) 
				SELECT ' + @columns + ', @info, GetDate() FROM deleted
				
				IF EXISTS (SELECT * FROM sys.columns WHERE OBJECT_NAME(object_id) = ''' + @mirrorName + ''' AND is_identity = 1)
					SET IDENTITY_INSERT [' + @schemaName + '].[' + @mirrorName + '] OFF'
		
		PRINT(@stmt)
		EXEC(@stmt)
		
		FETCH NEXT FROM @tableNames INTO @tableName
	END

	CLOSE @tableNames
	DEALLOCATE @tableNames
END
GO

-- RecoverRows
-- Procedure: Recovers all rows from the deletion log of the given table that match the given where clause
CREATE PROCEDURE RecoverRows
	@tableName SYSNAME,
	@where VARCHAR(MAX)
AS
BEGIN
	DECLARE @mirrorName SYSNAME = dbo.GetMirrorTableName(@tableName)
	DECLARE @schemaName SYSNAME = dbo.DL_SchemaName()
	DECLARE @columns VARCHAR(max) = dbo.GetColumns(@tableName)
	
	DECLARE @stmt VARCHAR(max)
	SET @stmt = '
		IF EXISTS (SELECT * FROM sys.columns WHERE OBJECT_NAME(object_id) = ''' + @tableName + ''' AND is_identity = 1)
			SET IDENTITY_INSERT [dbo].[' + @tableName + '] ON 
			
		INSERT INTO [dbo].[' + @tableName + '] (' + @columns + ')
			SELECT ' + @columns + ' FROM [' + @schemaName + '].[' + @mirrorName + ']
			WHERE ' + @where + ' 
			
		IF EXISTS (SELECT * FROM sys.columns WHERE OBJECT_NAME(object_id) = ''' + @tableName + ''' AND is_identity = 1)
			SET IDENTITY_INSERT [dbo].[' + @tableName + '] OFF 
			
		DELETE FROM [' + @schemaName + '].[' + @mirrorName + '] WHERE ' + @where
		
	PRINT(@stmt)
	EXEC(@stmt)
END
GO

-- RecoverAll
-- Procedure: Recovers all rows from all deletion log tables
CREATE PROCEDURE RecoverAll
AS
BEGIN
	DECLARE @tableNames CURSOR
	DECLARE @tableName SYSNAME
	
	SET @tableNames =
		CURSOR FOR
		SELECT Name
		FROM DL_DataTableNames()

	OPEN @tableNames
	FETCH NEXT FROM @tableNames INTO @tableName

	WHILE @@FETCH_STATUS = 0 BEGIN
		EXEC RecoverRows @tableName, '1=1'
		
		FETCH NEXT FROM @tableNames INTO @tableName
	END
END
GO

-- CleanupDeletionLog
-- Procedure: Cleans up all rows from the deletion log tables that are old enough (configurable by parameters)
CREATE PROCEDURE CleanupDeletionLog
	@datePart VARCHAR(max) = 'month',
	@diff INT = '3'
AS
BEGIN
	DECLARE @tableNames CURSOR
	DECLARE @tableName SYSNAME
	
	SET @tableNames =
		CURSOR FOR
		SELECT Name
		FROM DL_DeletionTableNames()

	OPEN @tableNames
	FETCH NEXT FROM @tableNames INTO @tableName
	
	DECLARE @schemaName SYSNAME = dbo.DL_SchemaName()

	WHILE @@FETCH_STATUS = 0 BEGIN
		DECLARE @stmt VARCHAR(MAX) = '
			DELETE FROM [' + @schemaName + '].[' + @tableName + ']
			WHERE DATEDIFF(' + @datePart + ',DeletionDate,GetDate()) > ' + CAST(@diff AS VARCHAR);
			
		PRINT(@stmt)
		EXEC(@stmt)
		
		FETCH NEXT FROM @tableNames INTO @tableName
	END
END
GO

EXEC SetupDeletionLogging
