PRINT 'CREATING DEPENDDB INDICES'

-- Indices for direct dependencies database query
-- Hint: No additional indices for indirect dependencies database query required

-- Assembly: Id & Name
-- JOIN [a].[Id] = [refdp].[AssemblyId] + WHERE [a].[Name] = @pX
-- Includes: Assembly information
CREATE NONCLUSTERED INDEX IX_Assembly_Name_Id ON Assembly ( Name, Id )
INCLUDE (PublicKeyToken, Version, Culture)
GO

-- DependencyPart: FullyQualifiedTypeNameHash & necessary includes
-- WHERE [refdp].[FullyQualifiedTypeNameHash] = @pX
-- Include FullyQualifiedTypeName: WHERE [refdp].[FullyQualifiedTypeName] = @pY
-- Include Id: Further querying
CREATE NONCLUSTERED INDEX IX_Type_FullyQualifiedTypeNameHash ON Type ( FullyQualifiedTypeNameHash )
INCLUDE ( FullyQualifiedTypeName, Id )
GO

-- DependencyPart: MemberSignatureHash & necessary includes
-- WHERE [refdp].[MemberSignatureHash] = @pX
-- Include MemberSignature: WHERE [refdp].[MemberSignature] = @pY
-- Include Id: Further querying
CREATE NONCLUSTERED INDEX IX_Member_MemberSignatureHash ON Member ( MemberSignatureHash )
INCLUDE ( MemberSignature, MemberType, Id )
GO

-- Indices from Database Engine Tuning Advisor
CREATE NONCLUSTERED INDEX IX_Dependency ON Dependency ( ReferencedPartId, DependentPartId, DependencyTypeId )
GO

CREATE NONCLUSTERED INDEX IX_DependencyPart_ID_MID_TID_AID ON DependencyPart (
	Id ASC,
	MemberId ASC,
	TypeId ASC,
	AssemblyId ASC
)
INCLUDE ( SourceFile, SourceLine )
GO

CREATE NONCLUSTERED INDEX IX_DependencyPart_TID_AID_ID_MID ON DependencyPart (
	TypeId ASC,
	AssemblyId ASC,
	Id ASC,
	MemberId ASC
)
INCLUDE ( SourceFile, SourceLine )
GO


-- Indices for database import

-- DependencyPart: MemberId, TypeId & AssemblyId
CREATE NONCLUSTERED INDEX IX_DependencyPart_MemberId_TypeId_AssemblyId ON DependencyPart ( MemberId, TypeId, AssemblyId )
GO

-- DependencyPart: TypeId & AssemblyId
CREATE NONCLUSTERED INDEX IX_DependencyPart_TypeId_AssemblyId ON DependencyPart ( TypeId, AssemblyId )
GO

-- DependencyPart: AssemblyId
CREATE NONCLUSTERED INDEX IX_DependencyPart_AssemblyID ON DependencyPart ( AssemblyId )
GO
