PRINT 'CREATING DEPENDDB DATABASE RETENTION TIME (Database Procedures and Functions)'
GO

-- Drop procedures
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'UpdateRetentionTime') AND type in (N'P')) DROP PROCEDURE UpdateRetentionTime;
GO

-- UpdateRetentionTime
-- Procedure: Updates the RetentionTimeStartDate for all project versions
CREATE PROCEDURE UpdateRetentionTime AS
BEGIN
	RAISERROR ('Updating retention time', 10, 1) WITH NOWAIT
	
	DECLARE @today DATETIME2 = DATEADD(day, DATEDIFF(day, 0, SYSDATETIME()), 0)
	
	-- Set RetentionTimeStartDate for all versions that are not the latest in their branch
	-- and that don't have a RetentionTimeStartDate yet
	UPDATE ProjectVersion
	SET RetentionTimeStartDate = @today 
	WHERE 
		RetentionTimeStartDate IS NULL AND 
		dbo.ProjectVersionIsLatestInBranch(Id) = 0
		
	-- Remove RetentionTimeStartDate for all versions that are latest in their branch
	UPDATE ProjectVersion 
	SET RetentionTimeStartDate = NULL 
	WHERE 
		dbo.ProjectVersionIsLatestInBranch(Id) = 1
END
GO
