PRINT 'CREATING DEPENDDB STRUCTURE (Database Tables)'

-- Drop tables
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'Dependency') AND type in (N'U')) DROP TABLE Dependency;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DependencyPart') AND type in (N'U')) DROP TABLE DependencyPart;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'Member') AND type in (N'U')) DROP TABLE Member;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'Type') AND type in (N'U')) DROP TABLE [Type];
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DependencyType') AND type in (N'U')) DROP TABLE DependencyType;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'DependencyCategory') AND type in (N'U')) DROP TABLE DependencyCategory;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'ProjectVersionAssembly') AND type in (N'U')) DROP TABLE ProjectVersionAssembly;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'Assembly') AND type in (N'U')) DROP TABLE Assembly;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'ProjectVersion') AND type in (N'U')) DROP TABLE ProjectVersion;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'ProjectBranch') AND type in (N'U')) DROP TABLE ProjectBranch
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'Project') AND type in (N'U')) DROP TABLE Project
GO

-- Project
CREATE TABLE Project (
	Id INT IDENTITY(1,1) NOT NULL,
	Name NVARCHAR(400) NOT NULL, -- 400 * 2 bytes = 800 bytes (maximum index size = 900 bytes)
	AutoClaimAssemblies BIT NOT NULL DEFAULT(0),
	
	CONSTRAINT PK_Project PRIMARY KEY ( Id ),
	CONSTRAINT UQ_Project_Name UNIQUE ( Name )
)

-- ProjectBranch
CREATE TABLE ProjectBranch (
	Id INT IDENTITY(1,1) NOT NULL,
	ProjectId INT NOT NULL,
	Name NVARCHAR(400) NOT NULL,
	
	CONSTRAINT PK_ProjectBranch PRIMARY KEY ( Id ),
	CONSTRAINT UQ_ProjectBranch_ProjectId_Name UNIQUE ( ProjectId, Name ),
	CONSTRAINT FK_ProjectBranch_Project FOREIGN KEY ( ProjectId ) REFERENCES Project ( Id )
)

-- ProjectVersion
CREATE TABLE ProjectVersion (
	Id INT IDENTITY(1,1) NOT NULL,
	ProjectBranchId INT NOT NULL,
	Version BINARY(8) NOT NULL,
	InsertDate DATETIME2 NOT NULL,
	SourceControlUrl NVARCHAR(MAX) NOT NULL,
	RetentionTimeStartDate DATETIME2 NULL,
	RetentionTimeInDays	INT NOT NULL,
	IsPinned BIT NOT NULL DEFAULT(0),
	RelevancyStatus TINYINT NULL,
	RelevancyStatusSourceProjectVersion INT NULL, -- if RelevancyStatus has been propagated by one or more dependent project versions, one of those is saved here for diagnostic reasons
	DeleteRequested BIT NOT NULL DEFAULT(0),
	
	CONSTRAINT PK_ProjectVersion PRIMARY KEY ( Id ),
	CONSTRAINT UQ_ProjectVersion_ProjectBranchId_Version UNIQUE ( ProjectBranchId, Version ),
	CONSTRAINT FK_ProjectVersion_ProjectBranch FOREIGN KEY ( ProjectBranchId ) REFERENCES ProjectBranch ( Id )
)

-- Assembly
CREATE TABLE Assembly (
	Id BIGINT IDENTITY(1,1) NOT NULL,
	Name NVARCHAR(400) NOT NULL,
	PublicKeyToken NVARCHAR(24) NOT NULL,
	Version NVARCHAR(MAX) NOT NULL,
	Culture NVARCHAR(MAX) NOT NULL,
	
	CONSTRAINT PK_Assembly PRIMARY KEY ( Id )
	-- Removed because index size would be bigger than 900 bytes
	-- , CONSTRAINT UQ_Assembly_Name_PublicKeyToken_Version_Culture UNIQUE ( Name, PublicKeyToken, Version, Culture )
)

-- ProjectVersionAssembly
CREATE TABLE ProjectVersionAssembly (
	ProjectVersionId INT NOT NULL,
	AssemblyId BIGINT NOT NULL,
	
	CONSTRAINT PK_ProjectVersionAssembly PRIMARY KEY ( ProjectVersionId, AssemblyId ),
	CONSTRAINT FK_ProjectVersionAssembly_ProjectVersion FOREIGN KEY ( ProjectVersionId ) REFERENCES ProjectVersion ( Id ),
	CONSTRAINT FK_ProjectVersionAssembly_Assembly FOREIGN KEY ( AssemblyId ) REFERENCES Assembly ( Id )
)

-- DependencyCategory
CREATE TABLE DependencyCategory (
	Id NVARCHAR(400) NOT NULL,
	TraverseIndirectDependencies BIT NOT NULL,
	TraverseInBothDirections BIT NOT NULL, -- required for mixins
	ShowDirectCountInSummary BIT NOT NULL, 
	ShowIndirectCountInSummary BIT NOT NULL,

	CONSTRAINT PK_DependencyCategory PRIMARY KEY ( Id )
)

-- DependencyType
CREATE TABLE DependencyType (
	Id INT IDENTITY(1,1) NOT NULL,
	Name NVARCHAR(400) NOT NULL,
	Category NVARCHAR(400) NOT NULL,
	ForwardDescription NVARCHAR(MAX) NOT NULL,
	BackwardDescription NVARCHAR(MAX) NOT NULL,
	
	CONSTRAINT PK_DependencyType PRIMARY KEY ( Id ),
	CONSTRAINT UQ_DependencyType_Name UNIQUE ( Name ),
	CONSTRAINT FK_DependencyType_DependencyCategory FOREIGN KEY ( Category ) REFERENCES DependencyCategory ( Id )
)

-- Type
CREATE TABLE Type (
    Id INT IDENTITY(1,1) NOT NULL,
    TypeType NVARCHAR(MAX) NOT NULL, -- Interface/Class/Struct/Enum
    FullyQualifiedTypeNameHash BINARY(16) NOT NULL,
    FullyQualifiedTypeName NVARCHAR(MAX) NOT NULL,
    GenericTypeParametersXML XML,

    CONSTRAINT PK_Type PRIMARY KEY ( Id )
)

-- Member
CREATE TABLE Member (
    Id INT IDENTITY(1,1) NOT NULL,
    MemberType NVARCHAR(MAX) NOT NULL, -- Method/Property/Event/Field
    MemberSignatureHash BINARY(16) NOT NULL,
    MemberSignature NVARCHAR(MAX) NOT NULL,
    MemberXML XML,

    CONSTRAINT PK_Member PRIMARY KEY ( Id )
)

-- DependencyPart
CREATE TABLE DependencyPart (
    Id BIGINT IDENTITY(1,1) NOT NULL,
    AssemblyId BIGINT NOT NULL,
    TypeId INT,
    MemberId INT,
    SourceFile NVARCHAR(MAX),
    SourceLine INT,

    CONSTRAINT PK_DependencyPart PRIMARY KEY ( Id ),
    CONSTRAINT FK_DependencyPart_Assembly_AssemblyId FOREIGN KEY ( AssemblyId ) REFERENCES Assembly ( Id ),
    CONSTRAINT FK_DependencyPart_Type_TypeId FOREIGN KEY ( TypeId ) REFERENCES Type ( Id ),
    CONSTRAINT FK_DependencyPart_Member_MemberId FOREIGN KEY ( MemberId ) REFERENCES Member ( Id )
)

-- Dependency
CREATE TABLE Dependency (
	DependencyTypeId INT NOT NULL,
	ReferencedPartId BIGINT NOT NULL,
	DependentPartId BIGINT NOT NULL,
	QueryRelevancy INT NULL,
	
	CONSTRAINT PK_Dependency PRIMARY KEY ( DependencyTypeId, ReferencedPartId, DependentPartId ),
	CONSTRAINT FK_Dependency_DependencyType FOREIGN KEY ( DependencyTypeId ) REFERENCES DependencyType ( Id ),
	CONSTRAINT FK_Dependency_ReferencedPartId_DependencyPart FOREIGN KEY ( ReferencedPartId ) REFERENCES DependencyPart ( Id ),
	CONSTRAINT FK_Dependency_DependentPartId_DependencyPart FOREIGN KEY ( DependentPartId ) REFERENCES DependencyPart ( Id )
)

GO
