PRINT 'CREATING DEPENDDB UTILS (Utility Database Procedures and Functions)'

-- IMPORTANT:
-- Do not use DDL statements in stored procedures
-- Do not use DML statements using Sch-M locks in stored procedures
-- Otherwise queries are going to timeout while running such stored procedures!

-- Drop functions
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'CreateTableFromXmlList') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION CreateTableFromXmlList;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'VersionToString') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION VersionToString;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'ProjectVersionIsLatestInBranch') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT')) DROP FUNCTION ProjectVersionIsLatestInBranch;
GO

-- CreateTableFromXmlList
-- Function: Converts the XML parameter into a table that can be used to join
CREATE FUNCTION CreateTableFromXmlList (@xmlList xml)
RETURNS TABLE
AS
RETURN (SELECT
			T.c.value('.', 'bigint') as Id
		FROM
			@xmlList.nodes('/L/V') T(c))
GO

-- VersionToString
-- Function: Converts a binary(8)-stored version to a human-readable string
CREATE FUNCTION VersionToString (@v BINARY(8))
RETURNS NVARCHAR(MAX)
AS
BEGIN
	DECLARE @res NVARCHAR(MAX)
	
	SET @res = CAST(CAST(@v AS BIGINT) / POWER(2,16) / POWER(2,16) / POWER(2,16) AS NVARCHAR) + '.' +
	           CAST(CAST(@v AS BIGINT) / POWER(2,16) / POWER(2,16) % POWER(2,16) AS NVARCHAR) + '.' +
			   CAST(CAST(@v AS BIGINT) / POWER(2,16) % POWER(2,16) AS NVARCHAR) + '.' +
			   CAST(CAST(@v AS BIGINT) % POWER(2,16) AS NVARCHAR)

	RETURN @res
END
GO

-- ProjectVersionIsLatestInBranch
-- Functions: Returns whether a project version is the latest version in its branch
CREATE FUNCTION ProjectVersionIsLatestInBranch(@projectVersionId INT)
RETURNS BIT
AS
BEGIN
	IF EXISTS (SELECT * FROM LatestProjectVersionByBranch WHERE ProjectVersionId = @projectVersionId)
		RETURN 1
		
	RETURN 0
END
GO