PRINT 'CREATING DEPENDDB VIEWS (Database Views)'

-- Drop views
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'ProjectInfoView') AND type in (N'V')) DROP VIEW ProjectInfoView;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'AssemblyProjectInfoView') AND type in (N'V')) DROP VIEW AssemblyProjectInfoView;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'ExclusiveAssembliesByProjectVersion') AND type in (N'V')) DROP VIEW ExclusiveAssembliesByProjectVersion;
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'LatestProjectVersionByBranch') AND type in (N'V')) DROP VIEW LatestProjectVersionByBranch;
GO

-- ProjectInfoView
CREATE VIEW ProjectInfoView AS
SELECT 
	pv.Id AS ProjectVersionId, p.Id AS ProjectId, p.Name AS ProjectName, 
	pb.Id AS BranchId, pb.Name AS BranchName, 
	pv.Version, dbo.VersionToString(pv.Version) AS VersionString, 
	pv.SourceControlUrl, pv.RelevancyStatus, pv.RelevancyStatusSourceProjectVersion
FROM ProjectVersion pv
INNER JOIN ProjectBranch pb ON pv.ProjectBranchId = pb.Id
INNER JOIN Project p ON pb.ProjectId = p.Id
GO

-- AssemblyProjectInfoView
CREATE VIEW AssemblyProjectInfoView AS
SELECT 
	pva.AssemblyId, pva.ProjectVersionId, 
	pi.ProjectId, pi.ProjectName, 
	pi.BranchId, pi.BranchName, 
	pi.Version, pi.SourceControlUrl,
	pi.RelevancyStatus, pi.RelevancyStatusSourceProjectVersion
FROM ProjectVersionAssembly pva
INNER JOIN ProjectInfoView pi ON pva.ProjectVersionId = pi.ProjectVersionId
GO

-- ExclusiveAssembliesByProjectVersion
CREATE VIEW ExclusiveAssembliesByProjectVersion AS
SELECT pva.ProjectVersionId, pva.AssemblyId
FROM ProjectVersionAssembly pva
WHERE NOT EXISTS (
	-- Filter those project versions which have assemblies that belong to another project version
	SELECT * FROM ProjectVersionAssembly ipva
	WHERE ipva.ProjectVersionId <> pva.ProjectVersionId
	AND ipva.AssemblyId = pva.AssemblyId
)
GO

-- LatestProjectVersionByBranch
CREATE VIEW LatestProjectVersionByBranch AS
SELECT sub.ProjectBranchId, sub.ProjectVersionId FROM (
	SELECT pv.ID AS ProjectVersionId, pv.ProjectBranchId as ProjectBranchId,
	ROW_NUMBER() OVER ( PARTITION BY pv.ProjectBranchId ORDER BY pv.Version DESC ) AS RowNumber
	FROM dbo.ProjectVersion pv
	WHERE pv.DeleteRequested = 0
	) AS sub
WHERE sub.RowNumber = 1
GO
